require 'securerandom'

module YourKeymaker
  class ApiKey
    LIFETIME = 60 * 24 * 60 * 60

    def revoke
      if self.class.storage[self.key]
        self.class.storage.delete(self.key)
      else
        raise RequestFailed.new(status: :not_found)
      end
      true
    end

    class << self
      def storage
        @storage ||= {}
      end

      def issue(user_id)
        key = SecureRandom.hex(13)
        storage[key] = user_id
        new(key: key, expires_at: Time.now + LIFETIME, user_id: user_id)
      end

      def lookup(key)
        if key = storage[key]
          new(key: key, user_id: storage[key])
        else
          raise RequestFailed.new(status: :not_found)
        end
      end

      def of(user_id)
        keys = storage.find_all{ |k, v| v == user_id }.map(&:first)
        keys.map do |key|
          new(key: key, expires_at: Time.now + LIFETIME, user_id: user_id)
        end
      end

      def revoke_all(user_id)
        storage.delete_if { |k, v| v == user_id }
        true
      end
    end
  end
end
